#include <math.h>
#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include "misc.h"

float centroid(float *ps, int r, int c, int cpeak, float wl, float bw, float *  wt)
{
  int i,j;
  float minps=1e99;
  float xc = 0.;
  float yc = 0.;

  float *sortit;
  sortit = (float *)malloc(r*sizeof(float));  

  for (j = cpeak - 1;j <  cpeak + 2; j++){
    for (i=0; i<r; i++){
      minps=MIN(minps,*(ps+j+i*c));
      sortit[i]=*(ps+j+i*c);
    }
    // Use Median instead.
      bubbleSort(sortit, r);
      minps = sortit[ (r / 2) -1 ];

    for (i = 0; i<r/2; i++){
      xc += (*(ps + j + i*c) - minps) *cos(i*2*PI/r)/minps;
      yc += (*(ps + j + i*c) - minps) *sin(i*2*PI/r)/minps;
      
      xc += ( *(ps + j + i*c + r/2*c) - minps) *cos(i*2*PI/r - PI)/minps;
      yc += ( *(ps + j + i*c + r/2*c) - minps) *sin(i*2*PI/r - PI)/minps;
    }
  }
  *wt=xc*xc+yc*yc / 3. ;  // Norm by number of sum-cols (3)
  free(sortit);
  return( (0.5*r/PI) *atan2(yc, xc) * wl*wl /bw );
}


float centroid_range(float *ps, int r, int c, int cpeak0, int cpeak1, 
              float wl, float bw, float *  wt)
{
  int i,j;
  float minps=1e99;
  float xc = 0.;
  float yc = 0.;

  float *sortit;
  sortit = (float *)malloc(r*sizeof(float));  

  for (j = cpeak0;j < cpeak1+1; j++){
    for (i=0; i<r; i++){
      minps=MIN(minps,*(ps+j+i*c));
      sortit[i]=*(ps+j+i*c);
    }
    // Use Median instead.
      bubbleSort(sortit, r);
      minps = sortit[ (r / 2) -1 ];

    for (i = 0; i<r/2; i++){
      xc += (*(ps + j + i*c) - minps) *cos(i*2*PI/r)/minps;
      yc += (*(ps + j + i*c) - minps) *sin(i*2*PI/r)/minps;
      
      xc += ( *(ps + j + i*c + r/2*c) - minps) *cos(i*2*PI/r - PI)/minps;
      yc += ( *(ps + j + i*c + r/2*c) - minps) *sin(i*2*PI/r - PI)/minps;
    }
  }
  *wt=xc*xc+yc*yc/ ( (float)(cpeak1-cpeak0+1));  // Norm by number of sum-cols
  free(sortit);
  return( (0.5*r/PI) *atan2(yc, xc) * wl*wl /bw );
}





/* only for 4 telescopes */
void bootStrapping(float w[], float OPD0[], float wMeanOPD[])
{
  float a, b, c, sum1, sum2;
  float OPD[6];
  int i;

  // w = [w21(0), w31(1), w41(2), w32(3), w42(4), w43(5)]
  // OPD = [OPD21, OPD31, OPD41, OPD32, OPD42, OPD43]
  // wMeanOPD = [wMeanOPD21, wMeanOPD31, wMeanOPD41]

  // zero out the OPDS for weak fringes.
  for (i=0;i<6;i++) {
    if (w[i] <= 3.0) OPD[i]=0.0; else OPD[i]=OPD0[i];
  }

  // OPD21  
  a = w[0];
  b = (w[1]+w[3]) <= 0. ? 0. : w[1]*w[3]/(w[1]+w[3]);
  c = (w[2]+w[4]) <= 0. ? 0. : w[2]*w[4]/(w[2]+w[4]);
  sum1 = a + b + c;
  sum2 = a*OPD[0] + b*(OPD[1] - OPD[3]) + c*(OPD[2] - OPD[4]);
  wMeanOPD[0] = (sum1 <= 0.) ? 0. :sum2/sum1;

  // OPD31
  a = w[1];
  b = (w[0]+w[3]) <= 0. ? 0. : w[0]*w[3]/(w[0]+w[3]);
  c = (w[5]+w[2]) <= 0. ? 0. : w[5]*w[2]/(w[5]+w[2]);
  sum1 = a + b + c;
  sum2 = a*OPD[1] + b*(OPD[0]+OPD[3]) + c*(OPD[2]-OPD[5]);
  wMeanOPD[1] = (sum1 <= 0.) ? 0. :sum2/sum1;

  // OPD41
  a = w[2];
  b = (w[0]+w[4]) <= 0. ? 0. : w[0]*w[4]/(w[0]+w[4]);
  c = (w[1]+w[5]) <= 0. ? 0. : w[1]*w[5]/(w[1]+w[5]);
  sum1 = a + b + c;
  sum2 = a*OPD[2] + b*(OPD[0]+OPD[4]) + c*(OPD[1]+OPD[5]);
  wMeanOPD[2] = (sum1 <= 0.) ? 0. :sum2/sum1;

}

/* only for 4 telescopes --  JDM 2005Sep11*/
void bootStrapping4(float w[], float OPD0[], float wMeanOPD[])
{
  float a11,a12,a13,a21,a22,a23,a31,a32,a33;
  float w12,w13,w14,w23,w24,w34;
  float opd12,opd13,opd14,opd23,opd24,opd34;
  float i11,i12,i13,i21,i22,i23,i31,i32,i33,deta;
  float b1,b2,b3;
  float OPD[6];
  int i;

  // w = [w21(0), w31(1), w41(2), w32(3), w42(4), w43(5)]
  // OPD = [OPD21, OPD31, OPD41, OPD32, OPD42, OPD43]
  // wMeanOPD = [wMeanOPD21, wMeanOPD31, wMeanOPD41]

   // zero out the OPDS for weak fringes.
  for (i=0;i<6;i++) {
    if (w[i] <= 1.0) {
      OPD[i]=0.0;
    } else OPD[i]=OPD0[i];
  }
  opd12=OPD[0];
  opd13=OPD[1];
  opd14=OPD[2];
  opd23=OPD[3];
  opd24=OPD[4];
  opd34=OPD[5];

  w12=MAX(w[0],.01);
  w13=MAX(w[1],.01);
  w14=MAX(w[2],.01);
  w23=MAX(w[3],.01);
  w24=MAX(w[4],.01);
  w34=MAX(w[5],.01);
  
  b1=opd12*w12-opd23*w23-opd24*w24;
  b2=opd13*w13+opd23*w23-opd34*w34;
  b3=opd14*w14+opd24*w24+opd34*w34;

  a11=w12+w23+w24;
  a12=-w23;
  a13=-w24;
  a21=a12;
  a22=w13+w23+w34;
  a23=-w34;
  a31=a13;
  a32=a23;
  a33=w14+w24+w34;

  i11=a33*a22-a32*a23;
  i12=-1*(a33*a12-a32*a13);
  i13=a23*a12-a22*a13;
  i21=-1*(a33*a21-a31*a23);
  i22=a33*a11-a31*a13;
  i23=-1*(a23*a11-a21*a13);
  i31=a32*a21-a31*a22;
  i32=-1*(a32*a11-a31*a12);
  i33=a22*a11-a21*a12;

  deta=a11*(a33*a22-a32*a23)-a21*(a33*a12-a32*a13)+a31*(a23*a12-a22*a13);

  wMeanOPD[0]=(i11*b1 +i12*b2 + i13*b3)/deta;
  wMeanOPD[1]=(i21*b1 +i22*b2 + i23*b3)/deta;
  wMeanOPD[2]=(i31*b1 +i32*b2 + i33*b3)/deta;

}


/* solve_3se */
// Solve 3 simulataneous equations
//
// y1 = a11 x1 + a12 x2 + a13 x3
// y2 = a21 x1 + a22 x2 + a23 x3
// y3 = a31 x1 + a32 x2 + a33 x3
//
// y1 = a1 x1 + b1 x2 + c1 x3
// y2 = a2 x1 + b2 x2 + c2 x3
// y3 = a3 x1 + b3 x2 + c3 x3
//
void solve_3se (double *a, double *b, double *c, double *y, double *x)
{
double a11,a21,a31,a12,a22,a32,a13,a23,a33,y1,y2,y3,x1,x2,x3;
double det3,m11,m12,m13,m21,m22,m23,m31,m32,m33;


a11=a[0];
a21=a[1];
a31=a[2];
a12=b[0];
a22=b[1];
a32=b[2];
a13=c[0];
a23=c[1];
a33=c[2];
y1=y[0];
y2=y[1];
y3=y[2];

det3=a11*(a33*a22-a32*a23)-a21*(a33*a12-a32*a13)+a31*(a23*a12-a22*a13);

m11=a33*a22-a32*a23;
m12=-(a33*a12-a32*a13);
m13=a23*a12-a22*a13;
m21=-(a33*a21-a31*a23);
m22=a33*a11-a31*a13;
m23=-(a23*a11-a21*a13);
m31=a32*a21-a31*a22;
m32=-(a32*a11-a31*a12);
m33=a22*a11-a21*a12;

x1=(m11*y1+m12*y2+m13*y3)/det3;
x2=(m21*y1+m22*y2+m23*y3)/det3;
x3=(m31*y1+m32*y2+m33*y3)/det3;

x[0]=x1;
x[1]=x2;
x[2]=x3;
}


/* champ-bs4  (bs=bootstrap)
// pass this program the weights and measured OPDS and this program
// will return an optimized set of delay line shifts (always assume delay0=0)
// same as MIRC.
// 
// the assumption is that 
// opd0 = delay0-delay_(n-1),
// opd1 = delay1-delay0
// opd2 = delay2-delay1
// etc.
// OPDS1 is the self-consistent best fit.
// one will need to write a 3, 5,6 telescope version of this for future work...
*/
//
void champ_bs4 (double *w, double *OPD0, double *OPD1, double *DL0 )
{
  double opd30,opd01,opd12,opd23,w30,w01,w12,w23;
  double a,b,c,ab,bc,y1,y2,y3,x1,x2,x3;
  double m11,m12,m13,m21,m22,m23,m31,m32,m33;
  double det3;
  double OPD[4];
  int i;
  
  // w = [w30), w01(1), w12(2), w23(3), w34(4), w43(5)]
  // OPD = [OPD30,OPD01, OPD12, OPD23 ] = [dl0-dl3,dl1-dl0,dl2-dl1,dl3-dl2]
  // DL0= [dl0,dl1,dl2,dl3]

   // zero out the OPDS for weak fringes.
   // could be done elsewhere..
  for (i=0;i<4;i++) {
    if (w[i] < 1.0) {
      OPD[i]=0.0;
    } else OPD[i]=OPD0[i];
  }
  opd30=OPD[0];
  opd01=OPD[1];
  opd12=OPD[2];
  opd23=OPD[3];
  
  w30=(double)MAX(w[0],.01);
  w01=(double)MAX(w[1],.01);
  w12=(double)MAX(w[2],.01);
  w23=(double)MAX(w[3],.01);

  a=w01+w12;
  b=w12+w23;
  c=w23+w30;

  ab=-1.0*w12;
  bc=-1.0*w23;
  y1=w01*opd01-w12*opd12;
  y2=w12*opd12-w23*opd23;
  y3=w23*opd23-w30*opd30;

det3=a*(c*b-bc*bc)-ab*(c*ab);

m11=c*b-bc*bc;
m12=-(c*ab);
m13=bc*ab;
m21=-(c*ab);
m22=c*a;
m23=-(bc*a);
m31=bc*ab;
m32=-(bc*a);
m33=b*a-ab*ab;

x1=(m11*y1+m12*y2+m13*y3)/det3;
x2=(m21*y1+m22*y2+m23*y3)/det3;
x3=(m31*y1+m32*y2+m33*y3)/det3;

DL0[0]=0.0; // by definition
DL0[1]=x1;
DL0[2]=x2;
DL0[3]=x3;
OPD1[0]=DL0[0]-DL0[3];
OPD1[1]=DL0[1]-DL0[0];
OPD1[2]=DL0[2]-DL0[1];
OPD1[3]=DL0[3]-DL0[2];

}





