/************************************************************************/
/* dlCommands.h								*/
/*                                                                      */
/* Modified from the JPL code to work under standard C.  		*/
/* Also contains isitCommand.h 						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar orignal by Brad Himes & K Young of JPL  */
/* Date   : NOVEMBER 1998                                               */
/************************************************************************/

#ifndef __DLCOMMANDS__
#define __DLCOMMANDS__

#include "dlTypes.h"
 
typedef struct 
{ 
   tIsitCmd		command;
   TARGET_TYPE		ttWhichTarget;
   BOOL			bTargetSetActive;
}tDlSetActiveTargetCmd;

typedef struct 
{ 
   tIsitCmd		command;
   BOOL bTurnDitherOn;
}tDlDitherOnCmd ;

typedef struct 
{
   tIsitCmd		command;
   BOOL bSuspendData;
}tDlDataRecordCmd;

typedef struct 
{ 
   tIsitCmd		command;
}tDlClearLaserCounterCmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlOffCmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlFrontCmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlBackCmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlTrackCmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlUser1Cmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlUser2Cmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlHomeCheckCmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlHomeSetCmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlDitherCalibrateCmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlDitherCheckCmd ;

typedef struct 
{ 
   tIsitCmd		command;
}tDlPingCmd ;

typedef struct 
{
  
   tIsitCmd		command;
   MOD_TYPE mtModulationType;
   double   dFrequency;
   double   dAmplitude;
   double   dMeanVal;
   OPLE_TIME     tSyncTime;

}tDlSetModulationCmd ;
 
typedef struct 
{
   
   tIsitCmd		command;

#ifndef SPARC
   int			spacer;	/* SPARC DOES WEIRD THINGS!!!*/
#endif
   double               dAmplitude;
   double               dPeriod;
}tDlSetDitherCmd ;
 
typedef struct 
{
    
   tIsitCmd		command;
#ifndef SPARC
    int    spacer1;     /* Sparc does odd boundry things */
#endif
   tGuiGsuDlServoParameters ServoParams;

}tGsuDlSetServoParametersCmd ;

typedef struct  
{
    tIsitCmd		command;
    int iDlGizmoId;
    tDlTargetBase Target;
}tInstSetManualTargetCmd ;

typedef struct  
{
    tIsitCmd		command;
    int iDlGizmoId;
    tDlTargetBase Target;
}tInstSetOpenLoopTargetCmd;

typedef struct 
{
    tIsitCmd		command;
    int iSecondsSinceZero;
}tInstSetTimeCmd;

typedef union ISIT_CMD {
   tIsitCmd                     genericCmd;
   tDlSetActiveTargetCmd        dlsatCmd;    /* DL Target src - */
   tDlDitherOnCmd               dldoCmd;     /* DL Dither On command */
   tDlDataRecordCmd             dldrCmd;     /* DL Data Record by KY 9/21/98 */
   tDlClearLaserCounterCmd      dlclcCmd;    /* DL clear laser ctr */
   tDlOffCmd                    dloCmd;      /* DL Off command */
   tDlFrontCmd                  dlfCmd;      /* DL Front command */
   tDlBackCmd                   dlbCmd;      /* DL Back command */
   tDlTrackCmd                  dltCmd;      /* DL Track command */
   tDlUser1Cmd                  dlu1Cmd;     /* DL User 1 command */
   tDlUser2Cmd                  dlu2Cmd;     /* DL User 2 command */
   tDlHomeCheckCmd              dlhcCmd;     /* DL Home Check command */
   tDlHomeSetCmd                dlhsCmd;     /* DL Home Set command */
   tDlDitherCalibrateCmd        dldcCmd;     /* DL Dither Calibrate command */
   tDlDitherCheckCmd            dldchkCmd;   /* DL Dither Check command */
   tDlSetDitherCmd          	dlsdCmd;     /* DL Set Dither command */
   tDlPingCmd                   dlpCmd;      /* DL Ping command */
   tDlSetModulationCmd          dlsmCmd;     /* DL Set Modulation command */
/*   tDlSetServoParametersCmd     dlsspCmd;    DL Set Servo Params */
   tGsuDlSetServoParametersCmd  gsudlsspCmd; /*   GSU DL Set Servo Params */
   tInstSetOpenLoopTargetCmd    instsoltCmd; /* Set Open Loop Target command*/
   tInstSetManualTargetCmd      instsmtCmd;  /* Set Manual Target command */
   tInstSetTimeCmd              inststCmd;   /* Instrument Set Time command */
/*   tExtintSetDataRecordingCmd   extsdrCmd;    Extint Set Data Recording */
} ISIT_CMD;

#endif
